@@ -95,7 +95,7 @@ class Repository(Resource[E], abc.ABC, typing.Generic[E]):
         raise NotImplementedError
 
 
-class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
+class SqlAlchemyRepository(Repository[E], abc.ABC, typing.Generic[E]):
     def __init__(self, session: orm.Session, /):
         self._session = session
         self._entity_type: typing.Optional[typing.Type[E]] = None
@@ -122,10 +122,9 @@ class SqlAlchemyRepository(Repository[E], typing.Generic[E]):
         return self.session.query(self.entity_type).get(item_id)
 
     @property
+    @abc.abstractmethod
     def entity_type(self) -> typing.Type[E]:
-        if self._entity_type is None:
-            self._entity_type = typing.get_args(self.__class__)[0]
-        return self._entity_type
+        raise NotImplementedError
 
     def rollback(self) -> None:
         self.session.rollback()
@@ -159,12 +158,12 @@ class DummyRepository(Repository[E], typing.Generic[E]):
     def __init__(
         self,
         *,
-        initial_values: typing.Iterable[E],
         key_fn: typing.Callable[[E], typing.Hashable],
+        initial_values: typing.Optional[typing.Iterable[E]] = None,
     ):
         super().__init__()
 
-        self._current_state: typing.List[E] = list(initial_values)
+        self._current_state: typing.List[E] = list(initial_values or [])
         self._previous_state: typing.List[E] = self._current_state.copy()
         self._key_fn = key_fn
 
@@ -205,7 +204,7 @@ class DummyRepository(Repository[E], typing.Generic[E]):
 
     def set_all(self, items: typing.Collection[E], /) -> typing.Collection[E]:
         self.events.append(("set_all", {"items": items}))
-        self._current_state = items
+        self._current_state = list(items)
         return items
 
     def update(self, item: E, /) -> E:
